#!/bin/bash

# slurm specific options for Marenostrum4
#SBATCH -p gpp
#SBATCH -q gp_debug
#SBATCH -A bsc32
#SBATCH -N 1
#SBATCH -n 3
#SBATCH -t 0:10:00
#SBATCH --ear=on
#SBATCH --ear-policy=MONITORING_ONLY
##SBATCH --ear-verbose=0
#SBATCH --constraint=perfparanoid
##SBATCH --exlusive

set -xuve

# setup pyOasis environment
CURRENT_PATH=$(pwd)
ROOTDIR=${CURRENT_PATH}/../../
export PYOASIS_ROOT=${ROOTDIR}/sources/oasis3-mct/generated
export LD_LIBRARY_PATH=${PYOASIS_ROOT}/lib:${LD_LIBRARY_PATH}
export PYTHONPATH=""
export PYTHONPATH=${PYOASIS_ROOT}/python:${PYTHONPATH}
export MPIRUN4PY="mpirun"

# Load modules
source ${PYOASIS_ROOT}/python/init.sh
source ${CURRENT_PATH}/mn5.env

# EAR
export SLURM_EAR_LOAD_MPI_VERSION="intel"
export EAR_LOAD_MPI_VERSION="intel"

# prepare rundir
rundir=run_ear/
srcdir=`pwd`
datadir=${ROOTDIR}/tests/data/ece3-toy-model

rm -rf $rundir
mkdir -p $rundir

for f in masks.nc grids.nc areas.nc; do
    cp -f $datadir/$f $rundir
done
cp -f $datadir/rmp* $rundir
cp -f $srcdir/*.py $rundir
cp -f $srcdir/namcouple $rundir
cp -f $srcdir/*.yaml $rundir

# Copy the restart files
cp -f $srcdir/rstas.nc $rundir
cp -f $srcdir/rstos.nc $rundir

cd $rundir

# run IFS+NEMO+runoff toy model
ifs_nproc=3
nemo_nproc=2
rnf_neproc=1

toy_model_script=$srcdir/toy_model.py

mpirun -np $ifs_nproc python3 $toy_model_script --yaml_file_path ifs_toy_model.yaml : -np $nemo_nproc python3 $toy_model_script --yaml_file_path nemo_toy_model.yaml : -np $rnf_neproc python3 $toy_model_script --yaml_file_path runoff_toy_model.yaml
