set -xuve

HPCROOTDIR=%HPCROOTDIR%
EXPID=%EXPID%
MEMBER=%MEMBER%
SDATE=%SDATE%

# Original launch_dir
launch_dir=${HPCROOTDIR}/python-amip-reader/sources/ece3-toy-model

# Create the directory with the initial data files needed by all members
inidata=${HPCROOTDIR}/${SDATE}/${MEMBER}/inidata
if [ -d ${inidata} ]; then
    rm -rf ${inidata}
fi
mkdir -p ${inidata}

# Copy the initial data files from the original location
original_inidata=${HPCROOTDIR}/python-amip-reader/tests/data/ece3-toy-model
# Copy the data files
for f in masks.nc grids.nc areas.nc rmp*; do
    cp -f ${original_inidata}/$f ${inidata}
done

# Copy toy model files (python and yaml) and namcouple configuration
cp -f ${launch_dir}/toy_model.py ${inidata}
cp -f ${launch_dir}/*.yaml ${inidata}
cp -f ${launch_dir}/namcouple ${inidata}

# Copy the Oasis restart files (rstos and rstas nc files)
cp -f ${launch_dir}/{rstos.nc,rstas.nc} ${inidata}

# TODO: Find a better way to determine the number of components
number_of_components=$(ls -1 ${inidata}/COMP*_toy_model.yaml | wc -l)

# Copy the scalability data
scalability_data_src_dir=${HPCROOTDIR}/python-amip-reader/sources/ece3-toy-model/ece3_scalability_data
for n in $(seq 1 ${number_of_components}); do
    cp -f ${scalability_data_src_dir}/COMP${n}_scalability.csv ${inidata}
done

# Set the corresponding scalability data file in the yaml files
for n in $(seq 1 ${number_of_components}); do
    yaml_config_file=${inidata}/COMP${n}_toy_model.yaml
    sed -i "s|scalability: .*|scalability: COMP${n}_scalability.csv|" ${yaml_config_file}
done

