set -xuve

# Autosubmit environment variables
EXPID=%EXPID%

SDATE=%SDATE%
LUCIA=%LUCIA%
nChunks=%NUMCHUNKS%
nMembers=%NUMMEMBERS%
SDATE=%SDATE%

HPCUSER=%HPCUSER%
HPCHOST=%HPCHOST%
DT_HOST=%DT_HOST%
DT_USER=%DT_USER%

HPCROOTDIR=%HPCROOTDIR%
ROOTDIR=%CURRENT_ROOTDIR%
PROJDEST=%PROJECT_DESTINATION%

# Need this to use dot "." separated floating points
export LC_NUMERIC="en_US.UTF-8"

# Declare some Global arrays
declare -A components_name
declare -A components_sypd
declare -A components_chsy
declare -A components_cpl_cost
declare -A components_nproc


get_performance_metrics () {
  declare -A acc_sypd
  declare -A acc_chsy
  declare -A acc_cpl_cost

  for (( chunk=1; chunk<=${nChunks}; chunk++ )); do
    table_file="${lucia_dir}/lucia_logs_${ci}_${mem}_${chunk}/lucia_results/table.csv"

    # Get the header of the table and convert it to an array with the column names
    table_header=$(head -n 1 ${table_file})
    IFS=',' read -ra table_columns <<< "$table_header"

    # Determine the total number of lines - 2 (header and Coupled)
    num_lines=$(wc -l < ${table_file})
  
    # Iterate through each component
    for ((line_number=2; line_number<=num_lines; line_number++)); do # Skip header keep the tail with the coupled run metrics
      # Read the line from the table file and convert it to an array
      current_line=$(sed "${line_number}q;d" "${table_file}")
      IFS=',' read -ra values <<< "$current_line"

      # Get the component name from the first column and store it in an array
      component_name=${values[0]}
      component_names+=("${component_name}")

      # Get the SYPD, CHSY and cpl_cost (partial coupling cost) for each component and accumulate the value for each chunk
      acc_sypd[${component_name}]=$(echo "${acc_sypd[${component_name}]} + ${values[8]}" | bc -l )
      acc_chsy[${component_name}]=$(echo "${acc_chsy[${component_name}]} + ${values[9]}" | bc -l )
      acc_cpl_cost[${component_name}]=$(echo "${acc_cpl_cost[${component_name}]} + ${values[10]}" | bc -l )
    done
  done

  # Compute the average of the Chunks for each component
  for component_name in "${component_names[@]}"; do
    components_sypd[${component_name}]=$(echo "${acc_sypd[${component_name}]} / ${nChunks}" | bc -l )
    components_chsy[${component_name}]=$(echo "${acc_chsy[${component_name}]} / ${nChunks}" | bc -l )
    components_cpl_cost[${component_name}]=$(echo "${acc_cpl_cost[${component_name}]} / ${nChunks}" | bc -l )
  done

  # Get the number of processes for each component by reading the nproc_config file
  source ${current_nproc_config_file}
  for component_name in "${component_names[@]}"; do
    components_nproc[${component_name}]=$(echo "${component_name}_nprocs_${ci}[@]")
  done
}


get_fitness (){
  # Loop on preevious iterations
  for (( iter=0; iter<=${ci}; iter++ )); do
    # Loop on all members
    for (( m=0; m<${nMembers}; m++ )); do  
      declare sypd=Coupled_SYPD_${iter}[${m}]
      declare chsy=Coupled_CHSY_${iter}[${m}]
      if [[ ${iter} -eq 0 ]] && [[ ${m} -eq 0 ]]; then
        max_sypd=${!sypd}
        min_sypd=${!sypd}
        max_chsy=${!chsy}
        min_chsy=${!chsy}
      else
        if (( $(echo "${!sypd} > ${max_sypd}" | bc -l) )); then
          max_sypd=${!sypd}
        fi 
        if (( $(echo "${!sypd} < ${min_sypd}" | bc -l) )); then
          min_sypd=${!sypd}
        fi
        if (( $(echo "${!chsy} > ${max_chsy}" | bc -l) )); then
          max_chsy=${!chsy}
        fi
        if (( $(echo "${!chsy} < ${min_chsy}" | bc -l) )); then
          min_chsy=${!chsy}
        fi
      fi
    done
  done

  # MinMax normalization for all tests and compute of the fitness
  # Loop on preevious iterations
  for (( iter=0; iter<=${ci}; iter++ )); do
    str_fitness=""
    # Loop on all members
    for (( m=0; m<${nMembers}; m++ )); do
      declare sypd=Coupled_SYPD_${iter}[${m}]
      declare chsy=Coupled_CHSY_${iter}[${m}]
      sypd_n=$(echo "(${!sypd} - ${min_sypd}) / (${max_sypd} - ${min_sypd})" | bc -l)
      chsy_n=$(echo "1 - (${!chsy} - ${min_chsy}) / (${max_chsy} - ${min_chsy})" | bc -l)
      f=$(echo "${sypd_n}*${TTS_ratio} + ${chsy_n}*${ETS_ratio}" | bc -l)
      str_fitness+="$(printf '%.2f' ${f}) "
    done
    # Save fitoness results into a file
    printf "fitness_${iter}=( ${str_fitness} )\n" > fitness_results_${iter}
    # Load the new fitness values
    source fitness_results_${iter}
  done
}


lookup_history () {
  is_match_with_previous_nproc_configuration=false

  # Check if the new candidate config was not assigned on the same iteration
  for (( m=0; m<${mem}; m++ )); do
    if [[ ${candidates_nproc_c1[$m]} -eq ${candidates_nproc_c1[$mem]} ]] && [[ ${candidates_nproc_c2[$m]} -eq ${candidates_nproc_c2[$mem]} ]]; then
      is_match_with_previous_nproc_configuration=true
    fi
  done

  # Check that this config was not assigned in an old lb iteration
  # Loop preevious iterations
  for (( iter=0; iter<=${ci}; iter++ )); do
    # Loop all members
    for (( m=0; m<${nMembers}; m++ )); do  
      previous_nproc_c1=${c1_name}_nprocs_${iter}[${m}]
      previous_nproc_c2=${c2_name}_nprocs_${iter}[${m}]
      # Check if the new coupled nproc config matches with any of the ones tryied before      
      if [[ ${candidates_nproc_c1[$mem]} -eq ${!previous_nproc_c1} ]] && [[ ${candidates_nproc_c2[$mem]} -eq ${!previous_nproc_c2} ]]; then
        is_match_with_previous_nproc_configuration=true
      fi
    done
  done
}


create_plot () {
  # Load all metrics
  for (( iter=0; iter<=${ci}; iter++ ))
  do
    source nproc_config_${iter}
    source performance_results_${iter}
    source fitness_results_${iter}
  done

  nodes=""
  edges=""
  for (( iter=0; iter<=${ci}; iter++ ))
  do
    for (( mem=0; mem<${nMembers}; mem++ ))
    do
      declare c1_np=${c1_name}_nprocs_${iter}[${mem}]
      declare c2_np=${c2_name}_nprocs_${iter}[${mem}]
      declare sypd=Coupled_SYPD_${iter}[${mem}]
      declare chsy=Coupled_CHSY_${iter}[${mem}]
      declare cpl_cost=Coupled_cost_${iter}[${mem}]
      declare fitness=fitness_${iter}[${mem}]
      if [[ ${iter} -eq ${max_iter} ]] && [[ ${mem} -eq ${max_mem} ]]; then
        node=$(cat << EOF
  node${iter}${mem} [label="{lb-iter ${iter}, test ${mem}|${c1_name}_nproc=${!c1_np}\n${c2_name}_nproc=${!c2_np}\n|SYPD=${!sypd}\nCHSY=${!chsy}\nCpl_cost=${!cpl_cost}\n|fitness=${!fitness}}", style=filled, color=darkolivegreen2];
EOF
        );
      else
        node=$(cat << EOF
  node${iter}${mem} [label="{lb-iter ${iter}, test ${mem}|${c1_name}_nproc=${!c1_np}\n${c2_name}_nproc=${!c2_np}\n|SYPD=${!sypd}\nCHSY=${!chsy}\nCpl_cost=${!cpl_cost}\n|fitness=${!fitness}}"];
EOF
        );
      fi

      nodes="$nodes${node}"$'\n'

      if [[ ${iter} -lt ${ci} ]]
      then
        edge="  node${iter}${mem} -> node$((iter+1))${mem}"
        edges="${edges}${edge}"$'\n'
      fi
    done
  done

  echo "digraph G {" > lb_graph.gv
  echo "  node [shape=record, fontsize=14];" >> lb_graph.gv
  echo "$nodes" >> lb_graph.gv
  echo "$edges" >> lb_graph.gv
  echo "}" >> lb_graph.gv

  dot -Tpng lb_graph.gv > graph.png
}


################################   MAIN   ##########################################

# Parameters
TTS_ratio=0.5
ETS_ratio=0.5
default_step=4
min_step=1
processes_per_node=48

cd ${ROOTDIR}

jobs_conf_file=${ROOTDIR}/conf/jobs_${EXPID}.conf
lucia_dir=${ROOTDIR}/lucia_logs_lb

# Get current.lb.iteration and the paths to the metrics
ci=`cat current.lb.iteration`
current_nproc_config_file="nproc_config_${ci}"
performance_metrics_file="performance_results_${ci}"

# Get the number of components
number_of_components=$(wc -l < ${current_nproc_config_file}})


# Next iteration
ni=$((ci+1))
new_nproc_config_file="nproc_config_${ni}"

# Previous iteration
pi=$((ci-1))

# Load performance values from previous iteration
for (( iter=0; iter<=${ci}; iter++ ))
do
  source nproc_config_${iter}
done

# Couter to store the number of members for which a new nproc configuration could not be found.
# If equals to nMembers --> exit auto-lb method
num_members_without_new_config=0
members_without_new_config=""
jobs_to_set_as_completed=""

# Main loop: Iterate over each one of the members
for (( mem=0; mem<${nMembers}; mem++ )); do

  get_performance_metrics

  # Set the step to use, based on the one used in the previous iteration by the same member
  if [[ $ci -eq 0 ]]; then
    step=${default_step}
  else
    # Get the current and previous nproc from the first componpent
    nproc_curr_config=${components_name[0]}_nprocs_${ci}[$mem]
    nproc_prev_config=${components_name[0]}_nprocs_${pi}[$mem]
    diff=$((${!nproc_curr_config} - ${!nproc_prev_config}))
    step=${diff#-}
  fi

  # Load nproc values from the current iteration, overwriting what we get from the table_file
  # Some processes are loss due to ElPin otherwise! (table values come from the MPI comm size)
  c1_nproc=${c1_name}_nprocs_${ci}[$mem]
  c2_nproc=${c2_name}_nprocs_${ci}[$mem]
  # Store them in an array for N components
  for (( component=0; component<${number_of_components}; component++ )); do
    declare component_name=${components_name[${component}]}
    declare component_nproc=${component_name}_nprocs_${ci}[$mem]
    declare ${component_name}_nprocs_${ci}[$mem]=${!component_nproc}
  done

  # Get the SYPD and CHSY of the current iteration
  declare Coupled_SYPD_${ci}[$mem]=$(printf '%.2f' ${cpl_SYPD})
  declare Coupled_CHSY_${ci}[$mem]=$(printf '%.0f' ${cpl_CHSY})
  declare Coupled_cost_${ci}[$mem]=$(printf '%.2f' ${cpl_cpl_cost})

  # Set the candidates to be the same as the previous iteration. This value is modifyied if a new nproc config has to be found
  candidates_nproc_c1[$mem]=${!c1_nproc}
  candidates_nproc_c2[$mem]=${!c2_nproc}

  #echo "cpl_cost comparison: $c1_cpl_cost >= $c2_cpl_cost ?"
  # While a new nproc configurations has not been assigned (done) and the step to use is greater than min_step,
  # look for new configurations not used in the past
  new_nproc_config_found=false
  while [[ "${new_nproc_config_found}" = false ]] && [[ ${step} -ge ${min_step} ]]; do
   #echo "c1_nproc form current iteration: "${!c1_nproc}
   #echo "c2_nproc form current iteration: "${!c2_nproc}
   if (( $(echo "${c1_cpl_cost} >= ${c2_cpl_cost}" | bc -l) )); then
      # Give resources from C1 to C2
      candidates_nproc_c1[$mem]=$((${!c1_nproc} - ${step}))
      candidates_nproc_c2[$mem]=$((${!c2_nproc} + ${step}))
    else
      # Give resources from C2 to C1
      candidates_nproc_c1[$mem]=$((${!c1_nproc} + ${step}))
      candidates_nproc_c2[$mem]=$((${!c2_nproc} - ${step}))
    fi

    # Check if the candidate nproc configuration has been already tested in the past
    lookup_history

    # If it is a new configuration, go on
    if [ "${is_match_with_previous_nproc_configuration}" = false ]; then
      declare ${c1_name}_nprocs_${ni}[$mem]=${candidates_nproc_c1[$mem]}
      declare ${c2_name}_nprocs_${ni}[$mem]=${candidates_nproc_c2[$mem]}
      new_nproc_config_found=true
    # If it is not a new configuration, reduce the step and try again
    else
      step=$(($step/2))
    fi
  done # While to find the new nproc config for the current member

  # If could NOT find a new configuration for this member, use the previous nproc config and increment the num_members_without_new_config counter
  if [[ "${new_nproc_config_found}" = false ]]; then
    echo "Could not find a new configuration for lb iteration $ni member $mem"
    declare ${c1_name}_nprocs_${ni}[$mem]=${!c1_nproc}
    declare ${c2_name}_nprocs_${ni}[$mem]=${!c2_nproc}
    num_members_without_new_config=$((num_members_without_new_config+1))
    members_without_new_config+="$mem "
    for ((chunk=1; chunk<=${nChunks}; chunk++)); do
      tmp_job_name=${EXPID}_${SDATE}_${MEMBERS_name[${mem}]}_${chunk}_SIM_${ni}_${mem} 
      jobs_to_set_as_completed="${jobs_to_set_as_completed}${tmp_job_name}   COMPLETED\n"
    done
    
  fi
done # Iterate over each one of the members


# Save the current SYPD and CHSY to a file
current_cpl_TTS=Coupled_SYPD_${ci}[@]
current_cpl_ETS=Coupled_CHSY_${ci}[@]
current_cpl_cost=Coupled_cost_${ci}[@]
current_performance_metrics_TTS="Coupled_SYPD_${ci}=( ${!current_cpl_TTS} )"
current_performance_metrics_ETS="Coupled_CHSY_${ci}=( ${!current_cpl_ETS} )"
current_performance_metrics_cpl_cost="Coupled_cost_${ci}=( ${!current_cpl_cost} )"
printf "${current_performance_metrics_TTS}\n${current_performance_metrics_ETS}\n${current_performance_metrics_cpl_cost}\n" > ${performance_metrics_file}


# Load all performance metrics before updating the fitness
for (( iter=0; iter<=${ci}; iter++ ))
do
  source performance_results_${iter}
done

# Update the fitness results with the new performance metrics
get_fitness

# If it was not possible to find at least 1 new configuration to test, print the best restult and stop the workflow
if [[ ${num_members_without_new_config} -eq ${nMembers} ]]; then
  echo ""
  echo "      ################################        "
  echo "Warning!! Not possible to find new configurations in any of the members. 
        Exiting the auto-lb method...."
  echo ""
  # Get the best config (max fitness) id (lb_iter and member)
  max_fitness=${fitness_0[0]}
  max_iter=0
  max_mem=0
  for ((iter=0; iter<=${ci}; iter++ )); do
    for ((mem=0; mem<${nMembers}; mem++)); do
      declare fitness=fitness_${iter}[${mem}]
      if (( $(echo "${!fitness} > ${max_fitness}" | bc -l) )); then
        max_fitness=${!fitness}
        max_iter=${iter}
        max_mem=${mem}
      fi
    done
  done

  # Get the best configuration metrics
  c1_best_nproc=${c1_name}_nprocs_${max_iter}[${max_mem}]
  c2_best_nproc=${c2_name}_nprocs_${max_iter}[${max_mem}]
  best_sypd=Coupled_SYPD_${max_iter}[${max_mem}]
  best_chsy=Coupled_CHSY_${max_iter}[${max_mem}]
  best_cpl_c=Coupled_cost_${max_iter}[${max_mem}]
  echo "----------------------------------------------------------------------------"
  echo "The best configuration has been found (TTS=${TTS_ratio}, ETS=${ETS_ratio}):"
  echo "${c1_name} nproc: ${!c1_best_nproc}" 
  echo "${c2_name} nproc: ${!c2_best_nproc}"
  echo "SYPD: ${!best_sypd}"
  echo "CHSY: ${!best_chsy}"
  echo "Cpl_cost: ${!best_cpl_c}"

  # Stop the workflow without stopping the experiment by setting the next iteration status to SUSPENDED
  for ((mem=0; mem<${nMembers}; mem++ )); do
    tmp_job_name=${EXPID}_${SDATE}_${MEMBERS_name[${mem}]}_1_SIM_${ni}_${mem}
    echo ${tmp_job_name} "SUSPENDED" >> pkl/updated_list_${EXPID}.txt
  done

  # Create a plot using dot (graphviz)
  create_plot

# If a new configuration has been found print it to a file and substitute the nprocs on the jobs.conf file
else
  # Save the next nproc configurations to a file
  next_c1_nprocs_arr=${c1_name}_nprocs_${ni}[@]
  next_c2_nprocs_arr=${c2_name}_nprocs_${ni}[@]
  new_nproc_config_c1="${c1_name}_nprocs_${ni}=( ${!next_c1_nprocs_arr} )"
  new_nproc_config_c2="${c2_name}_nprocs_${ni}=( ${!next_c2_nprocs_arr} )"
  printf "${new_nproc_config_c1}\n${new_nproc_config_c2}\n" > ${new_nproc_config_file}

  # Substitute the new nprocs to use in jobs.conf
  for ((mem=0; mem<${nMembers}; mem++));
  do
    next_c1_nproc=${c1_name}_nprocs_${ni}[${mem}]
    next_c2_nproc=${c2_name}_nprocs_${ni}[${mem}]

    # Set nprocs for each component
    sed -i "s/${c1_name}_nprocs_${ni}_${mem}/${!next_c1_nproc}/g" ${jobs_conf_file}
    sed -i "s/${c2_name}_nprocs_${ni}_${mem}/${!next_c2_nproc}/g" ${jobs_conf_file}

    # Set nprocs to allocate for the run
    total_nproc=$(( ${!next_c1_nproc} + ${!next_c2_nproc} + ${XIO_NUMPROC} + 1 ))
    sed -i "s/total_nproc_${ni}_${mem}/${total_nproc}/g" ${jobs_conf_file}

    # Set the number of nodes to allocate for the run
    total_nnodes=$(( (total_nproc + processes_per_node - 1) / processes_per_node ))
    sed -i "s/total_nnodes_${ni}_${mem}/${total_nnodes}/g" ${jobs_conf_file}
  done

  # Set jobs that whould repeat a configuration to completed
  printf "${jobs_to_set_as_completed}" > /esarchive/autosubmit/${EXPID}/pkl/updated_list_${EXPID}.txt

  ### Update the current iteration ###
  echo $ni > current.lb.iteration
fi

# Remove runtime directories
for ((mem=0; mem<${nMembers}; mem++));
do
  remove_runtime=true
  for member_as_completed in ${members_without_new_config[@]};
  do
    if [[ ${mem} -eq ${member_as_completed} ]];
    then
      remove_runtime=false
    fi
  done

  if ${remove_runtime};
  then
    $(ssh ${HPCUSER}@${HPCHOST} "rm -rf ${HPCROOTDIR}/${SDATE}/${MEMBERS_name[${mem}]}/runtime");
  fi
done
