set -xuve

# Autosubmit environment variables
EXPID=%EXPID%

SDATE=%SDATE%
LUCIA=%LUCIA%
nChunks=%NUMCHUNKS%
nMembers=%NUMMEMBERS%

HPCUSER=%HPCUSER%
HPCHOST=%HPCHOST%
DT_HOST=%DT_HOST%
DT_USER=%DT_USER%

HPCROOTDIR=%HPCROOTDIR%
ROOTDIR=%CURRENT_ROOTDIR%
PROJDEST=%PROJECT_DESTINATION%
PROJDIR=${ROOTDIR}/proj/${PROJDEST}


#####################################################################################################################
# Globals:
#   None
# Arguments:
#   None
# Returns:
#   None
# Purpose: Get results from HPC arch and store them in the workstation
#####################################################################################################################

function get_results_from_remote () {
for ((mem=0; mem<${nMembers}; mem++));
do
  for ((chunk=1; chunk<=${nChunks}; chunk++));
  do
    # Setup lucia and target dir
    if [ -f ${ROOTDIR}/current.lb.iteration ]; then
      ci=$(cat ${ROOTDIR}/current.lb.iteration)
      lucia_dir=${ROOTDIR}/lucia_logs_lb
      lucia_target_dir=${lucia_dir}/lucia_logs_${ci}_${mem}_${chunk}
    else
      lucia_dir=${ROOTDIR}/lucia_logs_scalability
      lucia_target_dir=${lucia_dir}/lucia_logs_${mem}_${chunk}
    fi

    # Remove lucia target dir if it exists
    if [[ -d ${lucia_target_dir} ]]; then
      rm -rf ${lucia_target_dir}
    fi
    mkdir -p ${lucia_target_dir}
    remote_dir=${HPCROOTDIR}/${SDATE}/$(printf 'fc%02d' ${mem})/runtime/log/$(printf '%03d' ${chunk})/
    rsync -avz ${HPCUSER}@${HPCHOST}:${remote_dir} ${lucia_target_dir}
    post_process_lucia_lite_logs "${lucia_target_dir}"
  done
done
}

#####################################################################################################################
# Globals:
#   None
# Arguments:
#   None
# Returns:
#   None
# Purpose: Post-process lucia logs if they exist, later CLEAN will transfer it to intermediate storage or esarchive
#####################################################################################################################

function post_process_logs() {
  if [ ${LUCIA} == -3 ]
  then
    post_process_lucia_lite_logs
  else
    echo "Lucia variable is ${LUCIA}. And should be -3 to run lucia-lite.py for the scalability metrics. Check
          the you proj.conf."
    exit 1
  fi
}


#####################################################################################################################
# Globals:
#   None
# Arguments:
#   None
# Returns:
#   None
# Purpose: Run the lucia-lite.py script to get the performance metrics 
#####################################################################################################################

function post_process_lucia_lite_logs() {
  # Get the lucia_target_dir from the argument
  lucia_target_dir=$1
  #check if there are Lucia logs present (either timeline_*.nc or lucia.??.000000)
  if ls ${lucia_target_dir}/timeline_*.nc > /dev/null 2>&1 || ls ${lucia_target_dir}/lucia.??.?????? > /dev/null 2>&1; then
    cd ${lucia_target_dir}
    lucia_script_file=${ROOTDIR}/auto_lb_workflow/lucia-alternative/lucia_lite_v2.py
    lucia_log=${lucia_target_dir}/lucia_logs.out
    data_path=${lucia_target_dir}
    method="average"
    ignore_init_ts=5
    ignore_end_ts=5
    num_ts_print=16
    components_to_ignore=""
    args=" --data_path ${data_path} ${method} ${ignore_init_ts} ${ignore_end_ts} ${num_ts_print} --components_to_ignore ${components_to_ignore}"
    echo "Starting lucia_lite analysis"
    python ${lucia_script_file} ${args} > ${lucia_log}
    cd ${ROOTDIR}
  else
    echo "No Lucia logs found in ${lucia_target_dir}"
    exit 1
  fi
}

### MAIN ###

# Load modules
module purge
module add matplotlib/3.2.2-foss-2015a-Python-3.7.3 
module add netcdf4-python/1.5.3-foss-2015a-Python-3.7.3
module add xarray/0.20.0-foss-2015a-Python-3.7.3 

# Get results from remote and execute Lucia-lite
get_results_from_remote
